#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  simul.py
#
#  Copyright 2024 john <jcoppens@vostro.ampr.net>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

# Cambios:
#
# 2024/03/13:
#   Por motivos de claridad:
#       - La carga del capacitor se guarda en la propiedad del capacitor en
#         Coulomb (no en voltaje como hicimos en clase)
#       - El valor de la resistencia y del capacitor se guarda en la
#         propiedad '.value'

import pylab as plt


DT = 0.01       # en segundos

class Resistor:
    def __init__(self, r_init):
        self.value = r_init

    def current(self, v):
        return v / self.value


class Capacitor:
    def __init__(self, c_init, v_init = 0):
        """ El constructor recibe como parámetros:
                c_init      Capacitancia (en F)
                v_init      Tensión inicial del capacitor (en V, por defecto 0V)
        """
        self.value = c_init
        self.charge_to(v_init)

    def voltage(self):
        """ Devuelve la tensión actual del capacitor
        """
        return self.charge / self.value

    def discharge(self, i, dt):
        """ 'Descarga' al capacitor con una corriente <i> durante un período
            de <dt> segundos.
            Devuelve además la tensión final.
        """
        self.charge -= i*dt
        return self.voltage()

    def charge_to(self, v):
        """ Pone la tensión del capacitor inmediatamente a 'v' voltios,
            aunque el valor guardado se convierte a Coulomb
        """
        self.charge = v * self.value


V_INIT = 100.0

r = Resistor(100000)
print('La corriente en ', r.value, 'Ohms, con 10V es', r.current(10), 'mA')

c = Capacitor(10e-6, V_INIT)
print('Carga inicial: ', c.voltage(), 'V (', c.charge, 'Coulomb)')

print('Descarga del capacitor en la resistencia:')
v = []  # lista de voltajes
x = []  # lista de tiempos
for t in range(200):
    v.append(c.voltage())
    x.append(DT*t)
    c.discharge(c.voltage()/r.value, DT)


plt.plot(x, v)
plt.grid(True)
plt.ylabel('Tensión sobre el capacitor (V)')
plt.xlabel('Tiempo (s)')
plt.title('Descarga de un capacitor de 10µF con 100kΩ)')
plt.show()

